import sys
from PyQt5 import QtWidgets, uic, Qt
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *

from modules.MainWindow import Ui_MainWindow
import modules.gui_interact as gui_interact
import modules.display as display


class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
    def __init__(self, *args, obj=None, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        self.setupUi(self)
        self.setvariables()
        self.setWindowTitle("Electrochemistry Parser v{}".format(self.version))
        self.connectEvents()
        display.initDisplay(self)
        display.readInputFromSave(self)

    # set global class variables
    def setvariables(self):
        self.version = '0.1.0'
        self.display_exists = True
        self.debug = False

    # Connect keystrokes and clicks to event functions
    def connectEvents(self):
        self.run_button.clicked.connect(self.runButtonClicked)
        self.table_button.clicked.connect(self.tableButtomClicked)
        self.plot_button.clicked.connect(self.plotButtonClicked)
        self.raw_button.clicked.connect(self.rawButtonClicked)

    def keyPressEvent(self,event): 
        if event.key()== Qt.Key_Return: 
            self.runButtonClicked()

    # Connection Functions 
    def runButtonClicked(self):
        gui_interact.runTasks(self)
    
    def tableButtomClicked(self):
        gui_interact.getTablePath(self)

    def plotButtonClicked(self):
        gui_interact.getFigurePath(self)
    
    def rawButtonClicked(self):
        gui_interact.getRawDataPath(self)
        
if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    window = MainWindow()
    window.show()
    app.exec_()


class rawDataError(Exception):
    def __init__(self):
        self.message = 'Please choose a valid raw data file!'
        super().__init__(self.message)

class ratioError(Exception):
    def __init__(self):
        self.message = 'Could not read Active:Carbon:Binder ratio input.'
        super().__init__(self.message)

class tablePermissionError(Exception):
    def __init__(self):
        self.message = 'Cannot save data table because the file is currently in use by another program.'
        super().__init__(self.message)

class figurePermissionError(Exception):
    def __init__(self):
        self.message = 'Cannot save figure because the file is currently in use by another program.'
        super().__init__(self.message)


from pkg_resources import parse_version
import requests
from requests.exceptions import HTTPError
import json
from numpy import round, isnan
import re
import electrochem.modules.errors as errors
# import modules.errors as errors

#Description: a safe rounding function
def safeRound(x, dec):
    try:
        rounded = round(x, dec)
        if isnan(x):
            return None
        else:
            return rounded
    except Exception:
        return x
# Description: Gets version data from Github
def compareVersion(version):
    try:
        response = requests.get(
        'https://api.github.com/repos/vince-wu/arbin_echem_tools/releases/latest'
        )
        json_response = response.json()
    except HTTPError:
        json_response = None
    except Exception:
        json_response = None
    if json_response:
        newestVersion = json_response['tag_name']
        newestVersionNumber = re.sub('[^0-9,.]','',newestVersion)
        if parse_version(version) == parse_version(newestVersionNumber):
            print('This program is up to date!')
            return 0
        elif parse_version(version) < parse_version(newestVersionNumber):
            return -1
        elif parse_version(version) > parse_version(newestVersionNumber):
            return 1
            print('Your program version is out of date! Please download the latest version from \
https://github.com/vince-wu/electrochemical-parsing/releases/latest.')

# Description gets the active mass of cathode material
def getActiveMass(mass, ACBratio):
    try:
        if not ACBratio:
            ratio = 1
        else:
            vals = [int(x) for x in ACBratio.split(':')]
            ratio = vals[0]/sum(vals)
    except Exception:
        raise errors.ratioError
    return ratio*mass


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.13.2
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1100, 950)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.textBrowser = QtWidgets.QTextBrowser(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textBrowser.sizePolicy().hasHeightForWidth())
        self.textBrowser.setSizePolicy(sizePolicy)
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout.addWidget(self.textBrowser)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label)
        self.label_6 = QtWidgets.QLabel(self.centralwidget)
        self.label_6.setObjectName("label_6")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.label_7 = QtWidgets.QLabel(self.centralwidget)
        self.label_7.setObjectName("label_7")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.label_8 = QtWidgets.QLabel(self.centralwidget)
        self.label_8.setObjectName("label_8")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_8)
        self.label_9 = QtWidgets.QLabel(self.centralwidget)
        self.label_9.setObjectName("label_9")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_9)
        self.system_input = QtWidgets.QLineEdit(self.centralwidget)
        self.system_input.setToolTip("")
        self.system_input.setObjectName("system_input")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.system_input)
        self.first_cycle_spinBox = QtWidgets.QSpinBox(self.centralwidget)
        self.first_cycle_spinBox.setToolTip("")
        self.first_cycle_spinBox.setMinimum(1)
        self.first_cycle_spinBox.setMaximum(1000000)
        self.first_cycle_spinBox.setProperty("value", 1)
        self.first_cycle_spinBox.setObjectName("first_cycle_spinBox")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.first_cycle_spinBox)
        self.num_cycles_spinBox = QtWidgets.QSpinBox(self.centralwidget)
        self.num_cycles_spinBox.setToolTip("")
        self.num_cycles_spinBox.setStatusTip("")
        self.num_cycles_spinBox.setMinimum(1)
        self.num_cycles_spinBox.setMaximum(100000)
        self.num_cycles_spinBox.setProperty("value", 10)
        self.num_cycles_spinBox.setObjectName("num_cycles_spinBox")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.num_cycles_spinBox)
        self.mass_doubleSpinBox = QtWidgets.QDoubleSpinBox(self.centralwidget)
        self.mass_doubleSpinBox.setToolTip("")
        self.mass_doubleSpinBox.setMinimum(0.0)
        self.mass_doubleSpinBox.setMaximum(1000000000.0)
        self.mass_doubleSpinBox.setProperty("value", 15.0)
        self.mass_doubleSpinBox.setObjectName("mass_doubleSpinBox")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.mass_doubleSpinBox)
        self.ratio_input = QtWidgets.QLineEdit(self.centralwidget)
        self.ratio_input.setToolTip("")
        self.ratio_input.setObjectName("ratio_input")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.ratio_input)
        self.horizontalLayout.addLayout(self.formLayout_2)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout.addWidget(self.line)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.label_5 = QtWidgets.QLabel(self.centralwidget)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.type_input = QtWidgets.QLineEdit(self.centralwidget)
        self.type_input.setObjectName("type_input")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.type_input)
        self.anode_input = QtWidgets.QLineEdit(self.centralwidget)
        self.anode_input.setObjectName("anode_input")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.anode_input)
        self.comments_input = QtWidgets.QPlainTextEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comments_input.sizePolicy().hasHeightForWidth())
        self.comments_input.setSizePolicy(sizePolicy)
        self.comments_input.setMinimumSize(QtCore.QSize(0, 0))
        self.comments_input.setPlainText("")
        self.comments_input.setObjectName("comments_input")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.comments_input)
        self.rate_comboBox = QtWidgets.QComboBox(self.centralwidget)
        self.rate_comboBox.setToolTip("")
        self.rate_comboBox.setEditable(True)
        self.rate_comboBox.setObjectName("rate_comboBox")
        self.rate_comboBox.addItem("")
        self.rate_comboBox.addItem("")
        self.rate_comboBox.addItem("")
        self.rate_comboBox.addItem("")
        self.rate_comboBox.addItem("")
        self.rate_comboBox.addItem("")
        self.rate_comboBox.addItem("")
        self.rate_comboBox.addItem("")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.rate_comboBox)
        self.horizontalLayout.addLayout(self.formLayout)
        self.line_2 = QtWidgets.QFrame(self.centralwidget)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout.addWidget(self.line_2)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.label_15 = QtWidgets.QLabel(self.centralwidget)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_17.addWidget(self.label_15)
        self.raw_inputfile = QtWidgets.QLineEdit(self.centralwidget)
        self.raw_inputfile.setText("")
        self.raw_inputfile.setObjectName("raw_inputfile")
        self.horizontalLayout_17.addWidget(self.raw_inputfile)
        self.raw_button = QtWidgets.QPushButton(self.centralwidget)
        self.raw_button.setObjectName("raw_button")
        self.horizontalLayout_17.addWidget(self.raw_button)
        self.verticalLayout_4.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.label_13 = QtWidgets.QLabel(self.centralwidget)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_15.addWidget(self.label_13)
        self.figure_path = QtWidgets.QLineEdit(self.centralwidget)
        self.figure_path.setText("")
        self.figure_path.setObjectName("figure_path")
        self.horizontalLayout_15.addWidget(self.figure_path)
        self.plot_button = QtWidgets.QPushButton(self.centralwidget)
        self.plot_button.setObjectName("plot_button")
        self.horizontalLayout_15.addWidget(self.plot_button)
        self.verticalLayout_4.addLayout(self.horizontalLayout_15)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.label_14 = QtWidgets.QLabel(self.centralwidget)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_16.addWidget(self.label_14)
        self.table_path = QtWidgets.QLineEdit(self.centralwidget)
        self.table_path.setText("")
        self.table_path.setObjectName("table_path")
        self.horizontalLayout_16.addWidget(self.table_path)
        self.table_button = QtWidgets.QPushButton(self.centralwidget)
        self.table_button.setObjectName("table_button")
        self.horizontalLayout_16.addWidget(self.table_button)
        self.verticalLayout_4.addLayout(self.horizontalLayout_16)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.run_button = QtWidgets.QPushButton(self.centralwidget)
        self.run_button.setObjectName("run_button")
        self.horizontalLayout_2.addWidget(self.run_button)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.horizontalLayout.addLayout(self.verticalLayout_4)
        self.horizontalLayout.setStretch(0, 1)
        self.horizontalLayout.setStretch(2, 1)
        self.horizontalLayout.setStretch(4, 5)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.verticalLayout_3.addLayout(self.horizontalLayout_11)
        self.verticalLayout.addLayout(self.verticalLayout_3)
        self.verticalLayout.setStretch(0, 10)
        self.verticalLayout.setStretch(1, 1)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.rate_comboBox.setCurrentIndex(2)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Electrochemical Parsing"))
        self.label.setToolTip(_translate("MainWindow", "The mass ratio between the active material, carbon, and binder"))
        self.label.setText(_translate("MainWindow", "Active:Carbon:Binder Ratio"))
        self.label_6.setToolTip(_translate("MainWindow", "The total cathode mass, including active material, carbon, and binder"))
        self.label_6.setText(_translate("MainWindow", "Cathode Mass (mg)"))
        self.label_7.setToolTip(_translate("MainWindow", "The number of charge/ discharge cycles to plot"))
        self.label_7.setText(_translate("MainWindow", "Number of Cycles to Plot"))
        self.label_8.setToolTip(_translate("MainWindow", "The cycle number you want to start plotting from. This will not affect calculations."))
        self.label_8.setText(_translate("MainWindow", "First Cycle to Plot"))
        self.label_9.setToolTip(_translate("MainWindow", "The name of your system. If left blank, a name will be generated based on your raw data filename"))
        self.label_9.setText(_translate("MainWindow", "System Name"))
        self.ratio_input.setText(_translate("MainWindow", "70:20:10"))
        self.label_2.setText(_translate("MainWindow", "Comments"))
        self.label_3.setText(_translate("MainWindow", "Anode Material"))
        self.label_4.setText(_translate("MainWindow", "Cell Type"))
        self.label_5.setText(_translate("MainWindow", "Cycling Rate"))
        self.type_input.setText(_translate("MainWindow", "Swagelok"))
        self.anode_input.setText(_translate("MainWindow", "Na Metal"))
        self.rate_comboBox.setItemText(0, _translate("MainWindow", "C/100"))
        self.rate_comboBox.setItemText(1, _translate("MainWindow", "C/50"))
        self.rate_comboBox.setItemText(2, _translate("MainWindow", "C/20"))
        self.rate_comboBox.setItemText(3, _translate("MainWindow", "C/10"))
        self.rate_comboBox.setItemText(4, _translate("MainWindow", "C/5"))
        self.rate_comboBox.setItemText(5, _translate("MainWindow", "C"))
        self.rate_comboBox.setItemText(6, _translate("MainWindow", "5C"))
        self.rate_comboBox.setItemText(7, _translate("MainWindow", "10C"))
        self.label_15.setToolTip(_translate("MainWindow", "The excel file that contains raw cycling data"))
        self.label_15.setText(_translate("MainWindow", "Raw Data"))
        self.raw_button.setText(_translate("MainWindow", "Choose File"))
        self.label_13.setToolTip(_translate("MainWindow", "The plot filename will be generated based of the system name"))
        self.label_13.setText(_translate("MainWindow", "Save Plot To"))
        self.plot_button.setText(_translate("MainWindow", "Choose Directory"))
        self.label_14.setToolTip(_translate("MainWindow", "The table filename will be generated based of the system name"))
        self.label_14.setText(_translate("MainWindow", "Save Table To"))
        self.table_button.setText(_translate("MainWindow", "Choose Directory"))
        self.run_button.setText(_translate("MainWindow", "Run"))



import pandas as pd
try:
    import pyodbc
except Exception:
    pass
import numpy as np

import matplotlib.pyplot as plt
    
import os.path
import tkinter as tk
from tkinter import filedialog
import csv
import re
from pkg_resources import parse_version
import requests
from requests.exceptions import HTTPError
import json
import os

from electrochem.modules.utils import safeRound, compareVersion, getActiveMass
from electrochem.modules import errors

# from modules.utils import safeRound, compareVersion, getActiveMass
# from modules import errors

# to upload: 
# change version numbers in _version.py, setup.cfg
# >>> python setup.py sdist
# >>> twine upload dist/*

# troubleshooting: 
# - can always delete /dist/ folder

class EchemData:
    def __init__(self, dischargeCapacity, chargeCapacity, power, avgVoltage, efficiency):
        self.dischargeCapacity = dischargeCapacity
        self.chargeCapacity = chargeCapacity
        self.power = power
        self.avgVoltage = avgVoltage
        self.efficiency = efficiency
    def __repr__(self):
        string = 'dischargeCapacity: %s\ncharge capacity: %s\npower: %s\naverage voltage: %s\nefficiency: \
%s\n'%(self.dischargeCapacity, self.chargeCapacity, self.power, self.avgVoltage, self.efficiency)
        return string

class DataSet:
    def __init__(self, name, mass, indexed_dfs):
        self.name = name
        self.mass = mass
        self.echem_df = indexed_dfs 
        self.properties = {}
    def addProperty(self, name, value):
        self.properties[name] = value

class ExtractedData:
    def __init__(self, firstCycleData, averageData, voltageRange):
        self.firstCycleData = firstCycleData
        self.averageData = averageData
        self.voltageRange = voltageRange

def parseArbin(source_path, save_path=None, table_name='Channel_Normal_Table'):
    #table_name = 'Channel_Normal_Table'
    # set up some constants
    MDB = source_path
    DRV = '{Microsoft Access Driver (*.mdb)}'
    PWD = 'pw'

    # connect to db
    con = pyodbc.connect('DRIVER={};DBQ={};PWD={}'.format(DRV,MDB,PWD))
    cur = con.cursor()
    header_row = []
    for r in cur.columns(table=table_name):
        # print(r.column_name)
        header_row.append(r.column_name)
        
    # run a query and get the results 
    SQL = 'SELECT * FROM %s;'%(table_name) # your query goes here
    rows = cur.execute(SQL).fetchall()
    rows.insert(0, header_row)
    cur.close()
    con.close()

    if save_path:
        with open(save_path, 'w', newline='') as f:
            csv_writer = csv.writer(f)
            csv_writer.writerows(rows)
            # print('saved file!')

# Description: parsed the excel file for Arbin cycler and extracts raw data into a Pandas dataframe
# Charge capacity data for cycle 'i' can be retrieved by:
# indexedData[i]['charge']['Charge_Capacity']
# Discharge capacity data for cycle 'i' can be retrieved by: 
# indexedData[i]['discharge']['Discharge_Capacity']
# Charge voltage data for cycle 'i' can be retrieved by: indexedData[i]['charge']['Voltage']
# Discharge voltage data for cycle 'i' can be retrieved by: indexedData[i]['discharge']['Voltage']
def toDataframe(path, active_mass):
    try:
        extension = os.path.splitext(path)[1]
        if extension == '.csv':
            echem_df = pd.read_csv(path)
        elif extension == '.xls':
            # Read excel file
            echem_df = pd.read_excel(path, sheet_name=1)
            # change column names for excel
            echem_df = echem_df.rename(columns={
                'Current(A)': 'Current', 
                'Charge_Capacity(Ah)': 'Charge_Capacity', 
                'Discharge_Capacity(Ah)': 'Discharge_Capacity',
                'Voltage(V)': 'Voltage',
                'Charge_Energy(Wh)': 'Charge_Energy',
                'Discharge_Energy(Wh)': 'Discharge_Energy',
                'dV/dt(V/s)': 'dV/dt',
                'Internal_Resistance(Ohm)': 'Internal_Resistance',
                'AC_Impedance(Ohm)': 'AC_Impedance',
                'ACI_Phase_Angle(Deg)': 'ACI_Phase_Angle',
                })
        echem_df = echem_df.sort_values(by=['Data_Point'])
        # Convert capacities into mAh/g
        echem_df['Charge_Capacity'] = echem_df['Charge_Capacity']*1000*1000/active_mass
        echem_df['Discharge_Capacity'] = echem_df['Discharge_Capacity']*1000*1000/active_mass
        # Parse data
        maxIndex = echem_df['Cycle_Index'].iloc[-1]
        indexedData = []
        for i in range(1, maxIndex+1):
            # Split data by Cycle Index
            df = echem_df[echem_df['Cycle_Index'] == i]
            # Remove soaking data
            df = df[df['Step_Index'] != 1] 
            # Remove tail data
            df = df[df['Step_Index'] != 4] 
            # Split data by charge
            charge_df = df[df['Current'] >= 0]
            # Split data by discharge
            discharge_df = df[df['Current'] < 0] 
            # Add data to list as a dictionary specifying charge/discharge  
            indexedData.append({'charge':charge_df, 'discharge':discharge_df})
    except OSError:
        raise errors.rawDataError()
    return indexedData

# Description: calculates electrochemical data on the first charge/ discharge curve.
def extractCycleEchem(indexedData, cycleIndex):
    chargeCapacities = indexedData[cycleIndex-1]['charge']['Charge_Capacity'].to_numpy()
    dischargeCapacities = indexedData[cycleIndex-1]['discharge']['Discharge_Capacity'].to_numpy()
    dischargeVoltages = indexedData[cycleIndex-1]['discharge']['Voltage'].to_numpy()
    if not dischargeCapacities.size == 0:
        dischargeCap = dischargeCapacities[-1]
        power = np.trapz(dischargeVoltages, dischargeCapacities, dx=0.01)
        avgVoltage = power/dischargeCap
    else:
        dischargeCap = None
        power = None
        avgVoltage = None
    if not chargeCapacities.size == 0:
        chargeCap = chargeCapacities[-1]
    else:
        chargeCap = None
    if chargeCap and dischargeCap:
        efficiency = dischargeCap/chargeCap*100
    else:
        efficiency = None
    echemData = EchemData(dischargeCap, chargeCap, power, avgVoltage, efficiency)
    return echemData

# Description: extracts the cycling voltage range
def extractVoltageRange(indexedData, cycleIndex):
    chargeVoltages = indexedData[cycleIndex-1]['charge']['Voltage'].to_numpy()
    dischargeVoltages = indexedData[cycleIndex-1]['discharge']['Voltage'].to_numpy()
    if not chargeVoltages.size == 0 and not dischargeVoltages.size == 0:
        min = dischargeVoltages[-1]
        max = chargeVoltages[-1]
    else:
        min = 'n/a'
        max = 'n/a'
    return [min, max]

def extractEchem(indexedData):
    avgVoltage = []
    avgDischargeCap = []
    avgChargeCap = []
    avgPower = []
    # Calculate single curve echem data for each cycle, then average out
    for i in range(len(indexedData)):
        data = extractCycleEchem(indexedData, i)
        if data.avgVoltage: 
            avgVoltage.append(data.avgVoltage)
        if data.dischargeCapacity:
            avgDischargeCap.append(data.dischargeCapacity)
        if data.chargeCapacity:
            avgChargeCap.append(data.chargeCapacity)
        if data.power:
            avgPower.append(data.power)
    avgEfficiency = np.mean(avgDischargeCap)/np.mean(avgChargeCap)*100
    avgData = EchemData(np.mean(avgDischargeCap), np.mean(avgChargeCap), np.mean(avgPower), 
    np.mean(avgVoltage), avgEfficiency)
    print('--------------------------------------------------------------------')
    print('***Average Electrochemical Data: ***')
    print('Charge Capacity [mAh/g]: %s'%(safeRound(avgData.chargeCapacity,2)))
    print('Discharge Capacity [mAh/g]: %s'%(safeRound(avgData.dischargeCapacity,2)))
    print('Average Discharge Voltage [V]: %s'%(safeRound(avgData.avgVoltage,2)))
    print('Discharge Power [Wh/kg]: %s'%(safeRound(avgData.power,2)))
    print('Coulombic Efficiency: %s%%'%(safeRound(avgData.efficiency,4)))
    print('--------------------------------------------------------------------')
    return avgData

def plotEchem(indexedData, figurePath, system, cycleList, show=True, molar_mass=0,  type='cap', style='standard'):
    option = 0
    plt.rcParams.update({'font.size': 25})
    plt.rcParams.update({'font.family':'Arial'})
    plt.figure(figsize=(8,6.5))
    fsize = 25  
    # if not __name__ == "__main__":
    #     #TODO: write case for package
    #     return
    ax = plt.gca()
    # For each cycle, plot both charge and discharge curves with the same color
    for index in cycleList:
        if index < len(indexedData) and index >= 0:
            if option == 0:
                x1_data = indexedData[index]['charge']['Charge_Capacity']
                x2_data = indexedData[index]['discharge']['Discharge_Capacity']
            elif option == 1:
                cap1_data = indexedData[index]['charge']['Charge_Capacity']
                cap2_data = indexedData[index]['discharge']['Discharge_Capacity']
                full_capacity = 2/molar_mass*96500*0.2777
                x1_data = 2 - cap1_data / full_capacity
                x2_data = 2 - cap1_data.iloc[-1]/full_capacity + cap2_data / full_capacity
            y1_data = indexedData[index]['charge']['Voltage']
            y2_data = indexedData[index]['discharge']['Voltage']
            color = next(ax._get_lines.prop_cycler)['color']
            plt.plot(x1_data, y1_data, '-', color=color,  linewidth=4, label='Cycle %s'%(index+1))
            plt.plot(x2_data, y2_data, '-', color=color, linewidth=4)
    # plt.ylim(1.3, 4.3)
    # plt.xlim(-2.7, 180)
    plt.ylabel('Voltage [V]', fontsize=fsize)
    #plt.title('Cycling Data for ' + system)
    #plt.title('Cycling Data for ' + 'Cathode A')
    if option == 1:
        plt.xlabel('$x$ in Na$_{2-x}$Mn$_3$(VO$_4$)$_2$PO$_4$', fontsize=fsize)
    elif option == 0:
        plt.xlabel('Capacity [mAh/g]', fontsize=fsize)
        full_capacity = 1/molar_mass*96500*0.2777
        ax2 = ax.secondary_xaxis('top')
        xticks = ax.get_xticks()
        ax2.set_xticks(xticks)
        ticklabels = [np.round(x/full_capacity, 1) for x in xticks]
        # ticklabels = [0, 0.5, 1.0, 1.5, 2, 2.5, 3]
        ax2.set_xticks([x*full_capacity for x in ticklabels])
        ax2.set_xticklabels(ticklabels)
        print([x*full_capacity for x in ticklabels])
        ax2.set_xlabel('Na$^+$ ions inserted/ extracted')
    plt.legend(prop={'size': 22}).set_draggable(True)
    plt.tight_layout()
    # Save matplotlib figure as png
    try:
        if figurePath:
            # print("saving figure")
            plt.savefig(figurePath + system + '.png')
            # if __name__ == "__main__":
            print('Plot saved to ', figurePath + system + '.png')
            print('--------------------------------------------------------------------')
    except PermissionError:
        raise errors.figurePermissionError
    if __name__ == "__main__":
        if show:
            plt.show()

def generateSummary(data, name, tablePath):
    with open(tablePath + '/' + name + '.csv', mode='w', newline='') as csvfile:
        csv_writer = csv.writer(csvfile, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
        for key, value in data.items():
            csv_writer.writerow([key, value])
    return

def generateEchemSummary(system, tablePath, indexedData, suppData):
    acc = 3
    data = {}
    voltageRange = extractVoltageRange(indexedData, 1)
    echemData = extractEchem(indexedData)
    cycleEchemData = extractCycleEchem(indexedData, 1)
    roundedRange = [safeRound(voltageRange[0], 3), safeRound(voltageRange[1], 3)]
    data['System'] = system
    data['Voltage Range [V]'] = str(roundedRange[0]) + '-' + str(roundedRange[1])
    data['Avg. Charge Cap. [mAh/g]'] = safeRound(echemData.chargeCapacity, acc)
    data['Avg. Discharge Cap. [mAh/g]'] = safeRound(echemData.dischargeCapacity, acc)
    data['Avg. Voltage [V]'] = safeRound(echemData.avgVoltage, acc)
    data['Avg. Power [Wh/kg]'] = safeRound(echemData.power, acc)
    data['Avg. Coulombic Efficiency'] = str(safeRound(echemData.efficiency, acc))+'%'
    data['First Charge Cap. [mAh/g]'] = safeRound(cycleEchemData.chargeCapacity, acc)
    data['First Discharge Cap. [mAh/g]'] = safeRound(cycleEchemData.dischargeCapacity,acc)
    data['First Discharge Volt. [V]'] = safeRound(cycleEchemData.avgVoltage, acc)
    data['First Discharge Power [Wh/kg]'] = safeRound(cycleEchemData.power, acc)
    data['First Discharge Efficiency'] = str(safeRound(cycleEchemData.efficiency, acc))+'%'
    data.update(suppData)
    generateSummary(data, system, tablePath)
    if __name__ == "__main__":
        print('Successfully saved data table!')

def getMass(sampleName):
    datapath = 'd:/Clement Research/Electrochem/cathode_masses.csv'
    with open(datapath, mode='r') as csvfile:
        csvreader = csv.reader(csvfile, delimiter=',')
        for row in csvreader:
            print(row[0])
            if row[0] == sampleName:
                return float(row[1])
    raise ValueError('Sample "{}" was not found in cathode_masses.csv!'.format(sampleName))

# Description: main function 
def runTasks(filepath, choosefile, figurePath, tablePath, cycleList, mass, 
    ACBratio, rate, cellType, anode, comments, molar_mass=0, active_ions=2, getMassFromFile=True):
    # Get system name
    filename = os.path.splitext(os.path.basename(filepath))[0]
    system = filename.replace('.xls', '').replace('VW-', '').replace('_', '.')
    # Get path
    if choosefile:
        # Prompt File Selection
        root = tk.Tk()
        root.withdraw()
        path = filedialog.askopenfilename()
    else:
        path = filepath
    extension = os.path.splitext(path)[1]
    if extension == '.res':
        parseArbin(path, 'data.csv', 'Channel_Normal_Table')
        path = 'data.csv'
    elif extension == '.csv' or '.xls':
        pass
    # Get active mass
    if getMassFromFile:
        mass = getMass(system)
    activeMass = getActiveMass(mass, ACBratio)
    # Parse data from excel file
    indexedData = toDataframe(path, activeMass)
    suppData = {
        'Mass': mass,
        'Active:Carbon:Binder Ratio': ACBratio,
        'Rate': rate,
        'Cell Type': cellType,
        'Anode': anode,
        'Comments': comments
    }
    # Generate CSV summary file
    generateEchemSummary(system, tablePath, indexedData, suppData)
    # Plot data
    plotEchem(indexedData, figurePath, system, cycleList, True, molar_mass)

if __name__ == "__main__":
    runTasks(
        # filepath = 'd:/Clement Research/Electrochem/VW-MnMn-B06-S02-E1.xls',
        # filepath = 'd:/Clement Research/Electrochem/VW-MnAl-B01-S01-B-E1.xls',
        # filepath = 'd:/Clement Research/Electrochem/VW-PMnMn-B02-S01-E1-01.xls',
        # filepath = 'd:/Clement Research/Electrochem/VW-PMnAl-B01-S01-B-E1.xls',
        filepath = 'd:/Clement Research/Electrochem/VW-MnMn-B10-S01-G1-E17.res',
        choosefile = False, # whether or not to choose your file directly (will ignore filepath, filename if true)
        figurePath = 'd:/Clement Research/Electrochem/Figures/', # the directory to save your plot in
        tablePath = 'd:/Clement Research/Electrochem/Summaries/', # the directory to save your table in
        cycleList = range(5),
        # mass = 28.5, # mass of entire cathode (do not multiply by active mass ratio)
        # mass = 9.7, #MnMn-B10-S01-G1-E3
        # mass = 11.4, #MnMn-B10-S01-G2-E3
        # mass = 10.4, #MnMn-B10-S01-G3-E3
        # mass = 9.2, #MnMn-B10-S01-G1-E4
        # mass = 11.1, #MnMn-B10-S01-G2-E4
        mass = 10.2, #MnMn-B10-S01-G3-E4
        ACBratio = '70:20:10', # active:carbon:binder ratio
        rate = 'C/20', # rate
        cellType = 'Swagelok', # cell type
        anode = 'Na', # anode material
        comments = None, # comments
        molar_mass = 555.61, #Na2Mn3(VO4)3
        # molar_mass = 527.65, #Na2Mn2Al(VO4)3
        # molar_mass = 495.71, #Na2Mn3(PO4)3
        # molar_mass = 467.75, #Na2Mn2Al(PO4)3
        # molar_mass = 535.64 #Na2Mn3(VO4)2PO4
        # molar_mass=494.1
        # molar_mass=490
        # molar_mass=484.4
        # molar_mass=473.3
        )



"""Top-level package for Arbin Electrochemical Tools."""
import electrochem._version

__author__ = """Vincent Wu"""
__email__ = 'vincentwu@ucsb.edu'
__version__ = __version__ = electrochem._version.__version__
from electrochem.parse import (parseArbin, toDataframe, extractEchem, 
extractCycleEchem, generateSummary, generateEchemSummary, plotEchem, extractVoltageRange)




from PyQt5 import QtWidgets, QtCore
from pkg_resources import parse_version
import sys
import matplotlib
matplotlib.use('Qt5Agg')
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure
import requests
from requests.exceptions import HTTPError
import json
import os
import re

def initDisplay(self):
    #Set initial display background color
    self.textBrowser.setStyleSheet("background-color: white; padding: 50px;")
    self.textBrowser.setAlignment(QtCore.Qt.AlignCenter)
    self.textBrowser.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse)
    versionText = getVersionText(self)
    #self.displayView.setBackgroundBrush(brush)
    txt = '\
    <div style="color: #202020;">\
        <h2 style="text-align: center">\
            Welcome to Electrochemical Parser v%s!\
        </h2>\
        <span style="text-align: center;font-size:10pt;">\
            <br>\
            Author: Vincent Wu (vincentwu@ucsb.edu)\
            <br>\
        </span>\
        <p style="text-align:left;font-size:10pt;">\
            This program tool allows researchers to quickly extract elecrochemical data and plot charge/\
            discharge curves from raw excel data files generated by the Arbin MITS Pro electrochemical cycler. \
            It is meant to speed up the data collection and analysis process for battery scientists. The tool \
            is easy to use and will generate both a customizable charge/ discharge plot and a summary table \
            containing information such as average voltage, capacities, Coulombic efficiencies, and more. \
            For more detailed instructions and to access the open-source code,\
            please refer to the project'"'"'s github page: \
            <a href="https://github.com/vince-wu/electrochemical-parsing">\
            https://github.com/vince-wu/electrochemical-parsing </a>.\
        </p>\
            %s\
    </div>'%(self.version, versionText)
    text = self.textBrowser.setText(txt)
    self.textBrowser.setOpenExternalLinks(True)

def getVersionText(self):
    color = '#000'
    boldcolor = '#00f'
    versionText = ''
    debug = False
    connected = True
    if debug:
        newestVersion = 'v2.0.0'
        bodyText = ''
    if not debug:
        #Get program's newest version from Github API, compare to current version
        json_response = getJson()
        # json_response = None
        if json_response:
            newestVersion = json_response['tag_name']
            bodyText = json_response['body']
            bodyText = re.sub(r'(<li>)', '<li> ', bodyText)
            bodyText = re.sub(r'(Changelog:)', '{} Changelog:'.format(newestVersion), bodyText)
            newestVersionNumber = re.sub('[^0-9,.]','',newestVersion)
        else:
            connected = False

    currentVersionNumber = re.sub('[^0-9,.]','',self.version)
    #Alert user based on if version is updated, ahead, or behind
    if connected == False:
        versionText = '\
        <p style="color: %s; font-size:10pt; text-align:center">\
            <br>\
            <span style="color: %s">\
                ***\
            </span>\
            To see if this program is up to date, please connect to the internet or \
            visit \
            <a style="color: " href="https://github.com/vince-wu/electrochemical-parsing/releases/latest">\
                this link\
            </a>\
            .\
            <span style="color: %s">\
                ***\
            </span>\
        </p>\
        '%(color, boldcolor, boldcolor)
    elif parse_version(newestVersionNumber) > parse_version(currentVersionNumber):
        versionText = '\
        <p style="color: %s; font-size:10pt; text-align:center">\
            <br>\
            <span style="color: %s">\
                ***\
            </span>\
            An updated version of this software \
            is available\
            <a style="color:" href="https://github.com/vince-wu/electrochemical-parsing/releases/latest">\
                here\
            </a>\
            !\
            <span style="color: %s">\
                ***\
            </span>\
        </p>\
        <p style="color: %s; font-size:9pt">\
            %s\
        </p>\
        '%(color, boldcolor, boldcolor, color, bodyText)

    elif parse_version(newestVersionNumber) == parse_version(currentVersionNumber):
        versionText = '\
        <p style="color: %s; font-size:10pt; text-align:center">\
            <br>\
            <span style="color: %s">\
                ***\
            </span>\
                This program version is up to date!\
            <span style="color: %s">\
                ***\
            </span>\
            <p style="color: %s; font-size:9pt">\
                %s\
            </p>\
        </p>\
        '%(color, boldcolor, boldcolor, color, bodyText)
    elif parse_version(newestVersionNumber) < parse_version(currentVersionNumber):
        versionText = '\
        <p style="color: %s; font-size:10pt; text-align:center">\
            <br>\
            <span style="color: %s">\
                ***\
            </span>\
                You are running an unpublished pre-release version of the program!  \
                Please report any bugs that you find to vincentwu@ucsb.edu.\
            <span style="color: %s">\
                ***\
            </span>\
            <p style="color: %s; font-size:9pt">\
                %s\
            </p>\
        </p>\
        '%(color, boldcolor, boldcolor, color, bodyText)
    return versionText

def getJson():
	try:
		response = requests.get(
			'https://api.github.com/repos/vince-wu/electrochemical-parsing/releases/latest'
			)
		json_response = response.json()
		return json_response
	except HTTPError as http_err:
		return None
	except Exception as err:
		return None


def readInputFromSave(self):
    if os.path.isfile('savefile.txt'):
        f = open('savefile.txt', 'r')
        lines = f.readlines()
        for l in lines:
            input = [x.strip() for x in l.split(',')]
            field = input[0]
            val = input[1]
            if field == 'initCycle':
                self.first_cycle_spinBox.setValue(int(val))
            elif field == 'numCycles':
                self.num_cycles_spinBox.setValue(int(val))
            elif field == 'mass':
                self.mass_doubleSpinBox.setValue(float(val))
            elif field == 'ratio':
                self.ratio_input.setText(val)
            elif field == 'rate':
                self.rate_comboBox.setCurrentText(val)
            elif field == 'type':
                self.type_input.setText(val)
            elif field == 'anode':
                self.anode_input.setText(val)
            elif field == 'dataPath':
                self.raw_inputfile.setText(val)
            elif field == 'figurePath':
                self.figure_path.setText(val)
            elif field == 'tablePath':
                self.table_path.setText(val)

class MplCanvas(FigureCanvasQTAgg):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = self.fig.add_subplot(111)
        self.gca = self.fig.gca()
        super(MplCanvas, self).__init__(self.fig)

def setUpGraph(self):
    dpi = 100
    height = self.textBrowser.frameGeometry().height()/dpi
    width = self.textBrowser.frameGeometry().width()/dpi
    self.graphWindow = MplCanvas(self, width, height, dpi)
    self.verticalLayout.insertWidget(0, self.graphWindow)
    self.verticalLayout.removeWidget(self.textBrowser)
    self.textBrowser.deleteLater()
    self.textBrowser = None
    self.display_exists = False

def displayError(self, text):
    msg = QtWidgets.QMessageBox()
    msg.setIcon(QtWidgets.QMessageBox.Warning)
    msg.setWindowTitle("Error")
    msg.setText(text)
    msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
    msg.exec_()


import csv, pyodbc

def read_arbin(source_path, save_path, table_name):
    #table_name = 'Channel_Normal_Table'
    # set up some constants
    MDB = source_path
    DRV = '{Microsoft Access Driver (*.mdb)}'
    PWD = 'pw'

    # connect to db
    con = pyodbc.connect('DRIVER={};DBQ={};PWD={}'.format(DRV,MDB,PWD))
    cur = con.cursor()
    header_row = []
    for r in cur.columns(table=table_name):
        header_row.append(r.column_name)
        
    # run a query and get the results 
    SQL = 'SELECT * FROM %s;'%(table_name) # your query goes here
    rows = cur.execute(SQL).fetchall()
    rows.insert(0, header_row)
    cur.close()
    con.close()

    with open(save_path, 'w', newline='') as f:
        csv_writer = csv.writer(f)
        csv_writer.writerows(rows)

read_arbin('MnMn-B02-S01-A-E1.res', 'test.csv', 'Channel_Normal_Table')


#!/usr/bin/env python

"""The setup script."""
from setuptools import setup, find_packages
import os

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=7.0', ]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest>=3', ]

name = "electrochem"

here = os.path.abspath(os.path.dirname(__file__))

version_ns = {}
with open(os.path.join(here, name, "_version.py")) as f:
    exec(f.read(), {}, version_ns)

setup(
    author="Vincent Wu",
    author_email='vincentwu@ucsb.edu',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Reads raw electrochemical cycling data and generates useful plots and tables for battery scientists.",
    entry_points={
        'console_scripts': [
            'electrochem=electrochem.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='electrochem',
    name='electrochem',
    packages=find_packages(include=['electrochem', 'electrochem.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/vince-wu/electrochem',
    version= version_ns["__version__"],
    zip_safe=False,
)



import os
import sys

import matplotlib
import requests
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure
from pkg_resources import parse_version
from PyQt5.QtWidgets import QApplication, QFileDialog
from requests.exceptions import HTTPError

import modules.errors
from modules.utils import getActiveMass
from modules.display import displayError, setUpGraph
from parse import toDataframe, parseArbin, generateEchemSummary, extractEchem

matplotlib.use('Qt5Agg')


class InputData():
    def __init__(self, dataPath, figurePath, tablePath, initCycle, numCycles, system, mass, ratio, 
    rate, type, anode, comments):
        self.dataPath = dataPath
        self.figurePath = figurePath
        self.tablePath = tablePath
        self.initCycle = initCycle
        self.numCycles = numCycles
        self.mass = mass
        self.ratio = ratio
        self. rate = rate
        self.type = type
        self.anode = anode
        self.comments = comments
        if system:
            self.system = system
        else:
            self.system = os.path.basename(dataPath).replace('.xls', '').replace('VW-', '')

# Reads the inputs in the GUI and assigns them to variables
def getInputs(self):
    system = self.system_input.text()
    initCycle = self.first_cycle_spinBox.value()
    numCycles = self.num_cycles_spinBox.value()
    mass = self.mass_doubleSpinBox.value()
    ratio = self.ratio_input.text()
    rate = self.rate_comboBox.currentText()
    type = self.type_input.text()
    anode = self.anode_input.text() 
    comments = self.comments_input.toPlainText()
    dataPath = self.raw_inputfile.text() 
    figurePath = self.figure_path.text() 
    tablePath = self.table_path.text() 
    inputs = InputData(dataPath, figurePath, tablePath, initCycle, numCycles, system, mass, ratio, 
    rate, type, anode, comments)
    return inputs

def saveInputs(self, inputs):
    f = open('savefile.txt', 'w')
    f.write('initCycle, %s\n'%(inputs.initCycle))
    f.write('numCycles, %s\n'%(inputs.numCycles))
    f.write('mass, %s\n'%(inputs.mass))
    f.write('ratio, %s\n'%(inputs.ratio))
    f.write('rate, %s\n'%(inputs.rate))
    f.write('type, %s\n'%(inputs.type))
    f.write('anode, %s\n'%(inputs.anode))
    f.write('dataPath, %s\n'%(inputs.dataPath))
    f.write('figurePath, %s\n'%(inputs.figurePath))
    f.write('tablePath, %s'%(inputs.tablePath))
    f.close()

def getRawDataPath(self):
    path = QFileDialog.getOpenFileName(self, 'Choose Raw Data File', '',"(*.xls *.csv *.res)")[0]
    if path:
        self.raw_inputfile.setText(path)

def getFigurePath(self):
    path = QFileDialog.getExistingDirectory(self, 'Select Directory for Plot Figure')
    if path:
        self.figure_path.setText(path)

def getTablePath(self):
    path = QFileDialog.getExistingDirectory(self, 'Select Directory for Data Table')
    if path:
        self.table_path.setText(path)

def plotData(self, partitioned_data, figurePath, system, initCycle, numCycles, show=True):
    if self.display_exists:
        setUpGraph(self)
    else:
        self.graphWindow.axes.cla()
        self.graphWindow.axes.clear()
        self.graphWindow.draw()
    fsize=14
    # Set the max cycle limit based on inputs
    maxCycle = initCycle + numCycles
    if maxCycle-1 > len(partitioned_data):
        maxCycle = len(partitioned_data) + 1
    if initCycle > len(partitioned_data):
        return
    # For each cycle, plot both charge and discharge curves with the same color
    for index in range(initCycle, maxCycle):
        x1_data = partitioned_data[index-1]['charge']['Charge_Capacity']
        y1_data = partitioned_data[index-1]['charge']['Voltage']
        x2_data = partitioned_data[index-1]['discharge']['Discharge_Capacity']
        y2_data = partitioned_data[index-1]['discharge']['Voltage']
        ax = self.graphWindow.gca
        color = next(ax._get_lines.prop_cycler)['color']
        self.graphWindow.axes.plot(x1_data, y1_data, '-', color=color, label='Cycle %s'%(index))
        self.graphWindow.axes.plot(x2_data, y2_data, '-', color=color)
    self.graphWindow.axes.tick_params(axis='both', which='major', labelsize=fsize)
    self.graphWindow.axes.set_xlabel('Capacity [mAh/g]', fontsize=fsize)
    self.graphWindow.axes.set_ylabel('Voltage [V]', fontsize=fsize)
    self.graphWindow.axes.set_title('Cycling Data for ' + system, fontsize=fsize)
    self.graphWindow.axes.legend(prop={'size': fsize})
    self.graphWindow.draw()
    # Save matplotlib figure as png
    if figurePath:
        self.graphWindow.fig.savefig(figurePath + '/' + system + '.png')
    if __name__ == "__main__":
        if show:
            self.graphWindow.axes.plot.show()
    QApplication.processEvents()

def runTasks(self):
    try:
        # Read inputs
        inputs = getInputs(self)
        # Get active mass
        activeMass = getActiveMass(inputs.mass, inputs.ratio)
        # Parse data from excel file
        self.statusbar.showMessage('Parsing raw data file...')
        QApplication.processEvents()
        extension = os.path.splitext(inputs.dataPath)[1]
        if extension == '.res':
            readArbin(inputs.dataPath, 'data.csv', 'Channel_Normal_Table')
            path = 'data.csv'
        else:
            path = inputs.dataPath
        partitioned_data = toDataframe(path, activeMass)
        # Extract electrochem data
        extractedData = extractEchem(partitioned_data)
        self.statusbar.showMessage('Parsing complete.', 2000)
        QApplication.processEvents()
        # Plot data
        plotData(self, partitioned_data, inputs.figurePath, inputs.system, inputs.initCycle, inputs.numCycles)
        # Generate CSV summary file
        generateEchemSummary(inputs.system, inputs.tablePath, extractedData.firstCycleData, extractedData.averageData, 
        extractedData.voltageRange, inputs.mass, inputs.ratio, inputs.rate, 
        inputs.type, inputs.anode, inputs.comments)
        # Save inputs
        saveInputs(self, inputs)
        self.statusbar.showMessage('Plot and data table saved successfully.', 3000)
    except Exception as e:
        if self.debug:
            raise e
        else:
            displayError(self, str(e))
